@@ -31,6 +31,7 @@ import shlex
 from beets.util import hidden
 import six
 from unidecode import unidecode
+from enum import Enum
 
 
 MAX_FILENAME_LENGTH = 200
@@ -124,6 +125,15 @@ class FilesystemError(HumanReadableException):
         return u'{0} {1}'.format(self._reasonstr(), clause)
 
 
+class MoveOperation(Enum):
+    """The file operations that e.g. various move functions can carry out.
+    """
+    MOVE = 0
+    COPY = 1
+    LINK = 2
+    HARDLINK = 3
+
+
 def normpath(path):
     """Provide the canonical form of the path suitable for storing in
     the database.
@@ -412,6 +422,8 @@ def syspath(path, prefix=True):
 
 def samefile(p1, p2):
     """Safer equality for paths."""
+    if p1 == p2:
+        return True
     return shutil._samefile(syspath(p1), syspath(p2))
 
 
