@@ -23,7 +23,7 @@ from beets import config
 from beets.autotag.hooks import AlbumInfo, TrackInfo, Distance
 from beets.plugins import BeetsPlugin
 from beets.util import confit
-from discogs_client import Release, Client
+from discogs_client import Release, Master, Client
 from discogs_client.exceptions import DiscogsAPIError
 from requests.exceptions import ConnectionError
 from six.moves import http_client
@@ -54,9 +54,11 @@ class DiscogsPlugin(BeetsPlugin):
             'apisecret': 'plxtUTqoCzwxZpqdPysCwGuBSmZNdZVy',
             'tokenfile': 'discogs_token.json',
             'source_weight': 0.5,
+            'user_token': '',
         })
         self.config['apikey'].redact = True
         self.config['apisecret'].redact = True
+        self.config['user_token'].redact = True
         self.discogs_client = None
         self.register_listener('import_begin', self.setup)
 
@@ -66,6 +68,12 @@ class DiscogsPlugin(BeetsPlugin):
         c_key = self.config['apikey'].as_str()
         c_secret = self.config['apisecret'].as_str()
 
+        # Try using a configured user token (bypassing OAuth login).
+        user_token = self.config['user_token'].as_str()
+        if user_token:
+            self.discogs_client = Client(USER_AGENT, user_token=user_token)
+            return
+
         # Get the OAuth token from a file or log in.
         try:
             with open(self._tokenfile()) as f:
@@ -205,11 +213,48 @@ class DiscogsPlugin(BeetsPlugin):
             self._log.debug(u"Communication error while searching for {0!r}",
                             query, exc_info=True)
             return []
-        return [self.get_album_info(release) for release in releases[:5]]
+        return [album for album in map(self.get_album_info, releases[:5])
+                if album]
+
+    def get_master_year(self, master_id):
+        """Fetches a master release given its Discogs ID and returns its year
+        or None if the master release is not found.
+        """
+        self._log.debug(u'Searching for master release {0}', master_id)
+        result = Master(self.discogs_client, {'id': master_id})
+        try:
+            year = result.fetch('year')
+            return year
+        except DiscogsAPIError as e:
+            if e.status_code != 404:
+                self._log.debug(u'API Error: {0} (query: {1})', e, result._uri)
+                if e.status_code == 401:
+                    self.reset_auth()
+                    return self.get_master_year(master_id)
+            return None
+        except CONNECTION_ERRORS:
+            self._log.debug(u'Connection error in master release lookup',
+                            exc_info=True)
+            return None
 
     def get_album_info(self, result):
         """Returns an AlbumInfo object for a discogs Release object.
         """
+        # Explicitly reload the `Release` fields, as they might not be yet
+        # present if the result is from a `discogs_client.search()`.
+        if not result.data.get('artists'):
+            result.refresh()
+
+        # Sanity check for required fields. The list of required fields is
+        # defined at Guideline 1.3.1.a, but in practice some releases might be
+        # lacking some of these fields. This function expects at least:
+        # `artists` (>0), `title`, `id`, `tracklist` (>0)
+        # https://www.discogs.com/help/doc/submission-guidelines-general-rules
+        if not all([result.data.get(k) for k in ['artists', 'title', 'id',
+                                                 'tracklist']]):
+            self._log.warn(u"Release does not contain the required fields")
+            return None
+
         artist, artist_id = self.get_artist([a.data for a in result.artists])
         album = re.sub(r' +', ' ', result.title)
         album_id = result.data['id']
@@ -218,33 +263,53 @@ class DiscogsPlugin(BeetsPlugin):
         # information and leave us with skeleton `Artist` objects that will
         # each make an API call just to get the same data back.
         tracks = self.get_tracks(result.data['tracklist'])
-        albumtype = ', '.join(
-            result.data['formats'][0].get('descriptions', [])) or None
-        va = result.data['artists'][0]['name'].lower() == 'various'
+
+        # Extract information for the optional AlbumInfo fields, if possible.
+        va = result.data['artists'][0].get('name', '').lower() == 'various'
+        year = result.data.get('year')
+        mediums = [t.medium for t in tracks]
+        country = result.data.get('country')
+        data_url = result.data.get('uri')
+
+        # Extract information for the optional AlbumInfo fields that are
+        # contained on nested discogs fields.
+        albumtype = media = label = catalogno = None
+        if result.data.get('formats'):
+            albumtype = ', '.join(
+                result.data['formats'][0].get('descriptions', [])) or None
+            media = result.data['formats'][0]['name']
+        if result.data.get('labels'):
+            label = result.data['labels'][0].get('name')
+            catalogno = result.data['labels'][0].get('catno')
+
+        # Additional cleanups (various artists name, catalog number, media).
         if va:
             artist = config['va_name'].as_str()
-        year = result.data['year']
-        label = result.data['labels'][0]['name']
-        mediums = len(set(t.medium for t in tracks))
-        catalogno = result.data['labels'][0]['catno']
         if catalogno == 'none':
-            catalogno = None
-        country = result.data.get('country')
-        media = result.data['formats'][0]['name']
+                catalogno = None
         # Explicitly set the `media` for the tracks, since it is expected by
         # `autotag.apply_metadata`, and set `medium_total`.
         for track in tracks:
             track.media = media
-            track.medium_total = mediums
-        data_url = result.data['uri']
+            track.medium_total = mediums.count(track.medium)
+            # Discogs does not have track IDs. Invent our own IDs as proposed
+            # in #2336.
+            track.track_id = str(album_id) + "-" + track.track_alt
+
+        # Retrieve master release id (returns None if there isn't one).
+        master_id = result.data.get('master_id')
+        # Assume `original_year` is equal to `year` for releases without
+        # a master release, otherwise fetch the master release.
+        original_year = self.get_master_year(master_id) if master_id else year
+
         return AlbumInfo(album, album_id, artist, artist_id, tracks, asin=None,
                          albumtype=albumtype, va=va, year=year, month=None,
-                         day=None, label=label, mediums=mediums,
-                         artist_sort=None, releasegroup_id=None,
+                         day=None, label=label, mediums=len(set(mediums)),
+                         artist_sort=None, releasegroup_id=master_id,
                          catalognum=catalogno, script=None, language=None,
                          country=country, albumstatus=None, media=media,
                          albumdisambig=None, artist_credit=None,
-                         original_year=None, original_month=None,
+                         original_year=original_year, original_month=None,
                          original_day=None, data_source='Discogs',
                          data_url=data_url)
 
@@ -287,7 +352,9 @@ class DiscogsPlugin(BeetsPlugin):
             # Only real tracks have `position`. Otherwise, it's an index track.
             if track['position']:
                 index += 1
-                tracks.append(self.get_track_info(track, index))
+                track_info = self.get_track_info(track, index)
+                track_info.track_alt = track['position']
+                tracks.append(track_info)
             else:
                 index_tracks[index + 1] = track['title']
 
@@ -305,30 +372,33 @@ class DiscogsPlugin(BeetsPlugin):
             # a 2-sided medium.
             if ''.join(m) in ascii_lowercase:
                 sides_per_medium = 2
-                side_count = 1  # Force for first item, where medium == None
 
         for track in tracks:
             # Handle special case where a different medium does not indicate a
             # new disc, when there is no medium_index and the ordinal of medium
             # is not sequential. For example, I, II, III, IV, V. Assume these
             # are the track index, not the medium.
+            # side_count is the number of mediums or medium sides (in the case
+            # of two-sided mediums) that were seen before.
             medium_is_index = track.medium and not track.medium_index and (
                 len(track.medium) != 1 or
-                ord(track.medium) - 64 != medium_count + 1
+                # Not within standard incremental medium values (A, B, C, ...).
+                ord(track.medium) - 64 != side_count + 1
             )
 
             if not medium_is_index and medium != track.medium:
-                if side_count < (sides_per_medium - 1):
-                    # Increment side count: side changed, but not medium.
-                    side_count += 1
-                    medium = track.medium
+                side_count += 1
+                if sides_per_medium == 2:
+                    if side_count % sides_per_medium:
+                        # Two-sided medium changed. Reset index_count.
+                        index_count = 0
+                        medium_count += 1
                 else:
-                    # Increment medium_count and reset index_count and side
-                    # count when medium changes.
-                    medium = track.medium
+                    # Medium changed. Reset index_count.
                     medium_count += 1
                     index_count = 0
-                    side_count = 0
+                medium = track.medium
+
             index_count += 1
             medium_count = 1 if medium_count == 0 else medium_count
             track.medium, track.medium_index = medium_count, index_count
@@ -354,15 +424,27 @@ class DiscogsPlugin(BeetsPlugin):
             """Modify `tracklist` in place, merging a list of `subtracks` into
             a single track into `tracklist`."""
             # Calculate position based on first subtrack, without subindex.
-            idx, medium_idx, _ = self.get_track_index(subtracks[0]['position'])
+            idx, medium_idx, sub_idx = \
+                self.get_track_index(subtracks[0]['position'])
             position = '%s%s' % (idx or '', medium_idx or '')
 
-            if len(tracklist) > 1 and not tracklist[-1]['position']:
-                # Assume the previous index track contains the track title, and
-                # "convert" it to a real track. The only exception is if the
-                # index track is the only one on the tracklist, as it probably
-                # is a medium title.
-                tracklist[-1]['position'] = position
+            if tracklist and not tracklist[-1]['position']:
+                # Assume the previous index track contains the track title.
+                if sub_idx:
+                    # "Convert" the track title to a real track, discarding the
+                    # subtracks assuming they are logical divisions of a
+                    # physical track (12.2.9 Subtracks).
+                    tracklist[-1]['position'] = position
+                else:
+                    # Promote the subtracks to real tracks, discarding the
+                    # index track, assuming the subtracks are physical tracks.
+                    index_track = tracklist.pop()
+                    # Fix artists when they are specified on the index track.
+                    if index_track.get('artists'):
+                        for subtrack in subtracks:
+                            if not subtrack.get('artists'):
+                                subtrack['artists'] = index_track['artists']
+                    tracklist.extend(subtracks)
             else:
                 # Merge the subtracks, pick a title, and append the new track.
                 track = subtracks[0].copy()
@@ -416,9 +498,10 @@ class DiscogsPlugin(BeetsPlugin):
         medium, medium_index, _ = self.get_track_index(track['position'])
         artist, artist_id = self.get_artist(track.get('artists', []))
         length = self.get_track_length(track['duration'])
-        return TrackInfo(title, track_id, artist, artist_id, length, index,
-                         medium, medium_index, artist_sort=None,
-                         disctitle=None, artist_credit=None)
+        return TrackInfo(title, track_id, artist=artist, artist_id=artist_id,
+                         length=length, index=index,
+                         medium=medium, medium_index=medium_index,
+                         artist_sort=None, disctitle=None, artist_credit=None)
 
     def get_track_index(self, position):
         """Returns the medium, medium index and subtrack index for a discogs
